//*******************************//
// Soldering Station based on
// Matthias Wagner
// www.k-pank.de/so
// Get.A.Soldering.Station@gmail.com
// by inghui
//*******************************//


#include <SPI.h>
#include <Wire.h>
#include <Adafruit_GFX.h>
#include <Adafruit_SSD1306.h>

#define OLED_RESET 4
Adafruit_SSD1306 display(OLED_RESET);

#if (SSD1306_LCDHEIGHT != 64)
#error("Height incorrect, please fix Adafruit_SSD1306.h!");
#endif

#define i 0.019 //0.0189    //constant current value

float r;
long integral=0; 
int igain; 


//#define STANDBYin A4
#define POTI   	A1
#define TEMPin 	A0
#define PWMpin 	10

#define CNTRL_GAIN 25

#define DELAY_MAIN_LOOP 	100
#define DELAY_MEASURE 		50
#define STANDBY_TEMP			100

#define OVER_SHOT 			2
#define MAX_PWM_LOW			160  //180
#define MAX_PWM_HI			180 //210		//254
#define MAX_POTI				400		//400Grad C

#define PWM_DIV 1						//default: 64   31250/64 = 2ms


long PWMval = 0; //pwm Out Val 0.. 255
int soll_temp = 300;
boolean standby_act = false;

void setup(void) {

 // Serial.begin(9600);

  pinMode(A0,INPUT);    // TEMPERATURE SENSOR
  pinMode(A1,INPUT);

  pinMode(PWMpin, OUTPUT);
  digitalWrite(PWMpin, LOW);
  setPwmFrequency(PWMpin, PWM_DIV);
  digitalWrite(PWMpin, LOW);

  display.begin(SSD1306_SWITCHCAPVCC, 0x3C);

  display.display();
  delay(1000);	

  // Clear the buffer.
  display.clearDisplay();

   // text display tests
  display.setTextSize(1);
  display.setTextColor(WHITE);
  display.setCursor(0,0);
  display.println("Soldering Station");
  display.display();

  delay(1000); 
  delay(1000); 
  delay(1000); 
  delay(1000); 
 
}

void loop() {

  //read actual temperature fromm constant current resistance measurement. 
	int actual_temperature = getTemperature();
  
  //read target temperature from Potentiomenter ADC
	soll_temp = map(analogRead(POTI), 0, 1024, 0, MAX_POTI);  // map adc val of POT to 0 -400 degree celcius	
	
	int soll_temp_tmp = soll_temp;         // temperature target

  //calculate different between target temperature and actual temperature
	int diff = (soll_temp_tmp)- actual_temperature;
	   
  integral+=diff;
  if(integral>60000) integral=60000; 
  else if (integral <-60000) integral = -60000;
  igain = integral*0.003;
      
  PWMval = diff*CNTRL_GAIN + igain ;
  
	
	int MAX_PWM;

	//Set max heating Power 
	MAX_PWM = actual_temperature <= STANDBY_TEMP ? MAX_PWM_LOW : MAX_PWM_HI;
	
	//8 Bit Range
 //if pwm >MAX_PWM then set to MAX_PWM 
 //else pwm negative? target temperature less than current temperature, set pwm =0, if pwm positive then maintain the pwm value from the control gain
	PWMval = PWMval > MAX_PWM ? PWMval = MAX_PWM : PWMval < 0 ? PWMval = 0 : PWMval;                                                                    
                                                                   
		
	//prevent overheating
	if (actual_temperature > 550){
		PWMval = 0;
		actual_temperature = 0;
	}
		
	analogWrite(PWMpin, PWMval);	
	
	displayHeating(soll_temp, actual_temperature, PWMval);
	
	delay(DELAY_MAIN_LOOP);		//wait for some time
}


int getTemperature()
{
  analogWrite(PWMpin, 0);		//switch off heater
  delay(DELAY_MEASURE);			//wait for some time (to get low pass filter in steady state)
  word adcT = analogRead(TEMPin); // read the input on analog pin 0:
  analogWrite(PWMpin, PWMval);	//switch heater back to last value

  float vR =(adcT*5.0)/1024.0;  // read voltage from constant current circuit of sensor
  //float r = vR/i - 46.1;        //convert to resistance value
  r = vR/i - 46.1;
  //float t=(r-32.437)/0.1517;    //convert to temperature value in degree celcius    
  float t=(r-28.937)/0.22;   
  return round(t); 
}





void displayHeating(int tempSOLL, int tempVAL, int pwmVAL){

  display.clearDisplay();
  display.setTextSize(1);
  display.setTextColor(WHITE);
  display.setCursor(0,0);
  display.print("Soldering Station");
  display.setCursor(0,24);
  display.print("Temperature");
  display.setCursor(0,32);
  display.print("Target:"); display.print(tempSOLL);
  display.setCursor(0,40);
  display.print("Actual:"); display.print(tempVAL);
  display.setCursor(0,48);
  display.print("PWM value:"); display.print(pwmVAL);
  //display.print("R value:"); display.print(r);
  display.setCursor(0,56);
  display.print("i value:"); display.print(igain);
  
  display.display();	
}


void setPwmFrequency(int pin, int divisor) {
  byte mode;
  if(pin == 5 || pin == 6 || pin == 9 || pin == 10) {
    switch(divisor) {
      case 1: mode = 0x01; break;
      case 8: mode = 0x02; break;
      case 64: mode = 0x03; break;
      case 256: mode = 0x04; break;
      case 1024: mode = 0x05; break;
      default: return;
    }
    if(pin == 5 || pin == 6) {
      TCCR0B = TCCR0B & 0b11111000 | mode;
    } else {
      TCCR1B = TCCR1B & 0b11111000 | mode;
    }
  } else if(pin == 3 || pin == 11) {
    switch(divisor) {
      case 1: mode = 0x01; break;
      case 8: mode = 0x02; break;
      case 32: mode = 0x03; break;
      case 64: mode = 0x04; break;
      case 128: mode = 0x05; break;
      case 256: mode = 0x06; break;
      case 1024: mode = 0x7; break;
      default: return;
    }
    TCCR2B = TCCR2B & 0b11111000 | mode;
  }
}
